package com.xdf.creative.module.controller;

import com.xdf.creative.base.controller.BaseController;
import com.xdf.creative.base.params.IdParam;
import com.xdf.creative.base.params.creative.SysOrganizeQueryParam;
import com.xdf.creative.base.vo.creative.SysOrganizeQueryVo;
import com.xdf.creative.module.entity.SysOrganize;
import com.xdf.creative.module.service.SysOrganizeService;
import com.xdf.creative.util.page.ApiResult;
import com.xdf.creative.util.page.PageTool;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;

/**
 * <p>
 * 组织信息 前端控制器
 * </p>
 *
 * @author DeanYe
 * @since 2019-11-05
 */
@Slf4j
@RestController
@RequestMapping("/sysOrganize")
@Api("组织信息 API")
public class SysOrganizeController extends BaseController {

    @Autowired
    private SysOrganizeService sysOrganizeService;


    /**
     * 添加组织信息
     */
    @PostMapping("/add")
    @ApiOperation(value = "添加SysOrganize对象", notes = "添加组织信息", response = ApiResult.class)
    public ApiResult<Boolean> addSysOrganize(@Valid @RequestBody SysOrganize sysOrganize) throws Exception {
        boolean flag = sysOrganizeService.save(sysOrganize);
        return ApiResult.result(flag);
    }

    /**
     * 修改组织信息
     */
    @PostMapping("/update")
    @ApiOperation(value = "修改SysOrganize对象", notes = "修改组织信息", response = ApiResult.class)
    public ApiResult<Boolean> updateSysOrganize(@Valid @RequestBody SysOrganize sysOrganize) throws Exception {
        boolean flag = sysOrganizeService.updateById(sysOrganize);
        return ApiResult.result(flag);
    }

    /**
     * 删除组织信息
     */
    @PostMapping("/delete")
    @ApiOperation(value = "删除SysOrganize对象", notes = "删除组织信息", response = ApiResult.class)
    public ApiResult<Boolean> deleteSysOrganize(@Valid @RequestBody IdParam idParam) throws Exception {
        boolean flag = sysOrganizeService.removeById(idParam.getId());
        return ApiResult.result(flag);
    }

    /**
     * 获取组织信息
     */
    @PostMapping("/getSysOrganize")
    @ApiOperation(value = "获取SysOrganize对象详情", notes = "查看组织信息", response = SysOrganizeQueryVo.class)
    public ApiResult<SysOrganizeQueryVo> getSysOrganize(@Valid @RequestBody IdParam idParam) throws Exception {
        SysOrganizeQueryVo sysOrganizeQueryVo = sysOrganizeService.getSysOrganizeById(idParam.getId());
        return ApiResult.ok(sysOrganizeQueryVo);
    }

    /**
     * 组织信息分页列表
     */
    @PostMapping("/getPageList")
    @ApiOperation(value = "获取SysOrganize分页列表", notes = "组织信息分页列表", response = SysOrganizeQueryVo.class)
    public ApiResult<PageTool<SysOrganizeQueryVo>> getSysOrganizePageList(@Valid @RequestBody SysOrganizeQueryParam sysOrganizeQueryParam) throws Exception {
        PageTool<SysOrganizeQueryVo> pageList = sysOrganizeService.getSysOrganizePageList(sysOrganizeQueryParam);
        return ApiResult.ok(pageList);
    }

}

