package com.xdf.creative.module.controller;

import com.xdf.creative.base.controller.BaseController;
import com.xdf.creative.base.params.IdParam;
import com.xdf.creative.base.params.creative.SysRegionQueryParam;
import com.xdf.creative.base.vo.creative.SysRegionQueryVo;
import com.xdf.creative.module.entity.SysRegion;
import com.xdf.creative.module.service.SysRegionService;
import com.xdf.creative.util.page.ApiResult;
import com.xdf.creative.util.page.PageTool;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

/**
 * <p>
 * 全国行政区域[sys_region] 前端控制器
 * </p>
 *
 * @author DeanYe
 * @since 2019-11-06
 */
@Slf4j
@RestController
@RequestMapping("/sysRegion")
@Api("全国行政区域[sys_region] API")
public class SysRegionController extends BaseController {

    @Autowired
    private SysRegionService sysRegionService;

    /**
     * 添加全国行政区域[sys_region]
     *
     * @param sysRegion
     * @return
     * @throws Exception
     */
    @PostMapping("/add")
    @ApiOperation(value = "添加SysRegion对象", notes = "添加全国行政区域[sys_region]", response = ApiResult.class)
    public ApiResult<Boolean> addSysRegion(@Valid @RequestBody SysRegion sysRegion) throws Exception {
        boolean flag = sysRegionService.save(sysRegion);
        return ApiResult.result(flag);
    }

    /**
     * 修改全国行政区域[sys_region]
     */
    @PostMapping("/update")
    @ApiOperation(value = "修改SysRegion对象", notes = "修改全国行政区域[sys_region]", response = ApiResult.class)
    public ApiResult<Boolean> updateSysRegion(@Valid @RequestBody SysRegion sysRegion) throws Exception {
        boolean flag = sysRegionService.updateById(sysRegion);
        return ApiResult.result(flag);
    }

    /**
     * 删除全国行政区域[sys_region]
     */
    @PostMapping("/delete")
    @ApiOperation(value = "删除SysRegion对象", notes = "删除全国行政区域[sys_region]", response = ApiResult.class)
    public ApiResult<Boolean> deleteSysRegion(@Valid @RequestBody IdParam idParam) throws Exception {
        boolean flag = sysRegionService.removeById(idParam.getId());
        return ApiResult.result(flag);
    }

    /**
     * 获取全国行政区域[sys_region]
     */
    @PostMapping("/info")
    @ApiOperation(value = "获取SysRegion对象详情", notes = "查看全国行政区域[sys_region]", response = SysRegionQueryVo.class)
    public ApiResult<SysRegionQueryVo> getSysRegion(@Valid @RequestBody IdParam idParam) throws Exception {
        SysRegionQueryVo sysRegionQueryVo = sysRegionService.getSysRegionById(idParam.getId());
        return ApiResult.ok(sysRegionQueryVo);
    }

    /**
     * 全国行政区域[sys_region]分页列表
     */
    @GetMapping("/getPageList")
    @RequiresPermissions("/sysRegion/getPageList")
    @ApiOperation(value = "获取SysRegion分页列表", notes = "全国行政区域[sys_region]分页列表", response = SysRegionQueryVo.class)
    public ApiResult<PageTool<SysRegionQueryVo>> getSysRegionPageList(@Valid @RequestBody SysRegionQueryParam sysRegionQueryParam) throws Exception {
        PageTool<SysRegionQueryVo> pageList = sysRegionService.getSysRegionPageList(sysRegionQueryParam);
        return ApiResult.ok(pageList);
    }


    /**
     * 全国行政区域[sys_region]分页列表
     */
    @PostMapping("/getRegionList")
    @RequiresPermissions("/sysRegion/getRegionList")
    @ApiOperation(value = "获取SysRegion列表", notes = "全国行政区域[sys_region]分页列表", response = SysRegionQueryVo.class)
    public ApiResult<List<SysRegionQueryVo>> getSysRegionList(@Valid @RequestBody SysRegionQueryParam sysRegionQueryParam) throws Exception {
        return ApiResult.ok(sysRegionService.getSysRegionList(sysRegionQueryParam));
    }

}

