
package com.xdf.creative.module.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xdf.creative.base.params.creative.AddSysRoleParam;
import com.xdf.creative.base.params.creative.SysRoleQueryParam;
import com.xdf.creative.base.params.creative.UpdateSysRoleParam;
import com.xdf.creative.base.service.impl.BaseServiceImpl;
import com.xdf.creative.base.vo.creative.SysRoleQueryVo;
import com.xdf.creative.enums.StateEnum;
import com.xdf.creative.enums.StatusCode;
import com.xdf.creative.module.entity.SysRole;
import com.xdf.creative.module.entity.SysUserRole;
import com.xdf.creative.module.mapper.SysRoleMapper;
import com.xdf.creative.module.service.*;
import com.xdf.creative.support.exception.BusinessException;
import com.xdf.creative.support.exception.DaoException;
import com.xdf.creative.util.convert.SysRoleConvert;
import com.xdf.creative.util.page.ApiResult;
import com.xdf.creative.util.page.PageTool;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.SetUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.Serializable;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Set;


/**
 * <pre>
 * 系统角色 服务实现类
 * </pre>
 *
 * @author DeanYe
 * @since 2019-10-24
 */
@Slf4j
@Service
public class SysRoleServiceImpl extends BaseServiceImpl<SysRoleMapper, SysRole> implements SysRoleService {

    @Autowired
    private SysRoleMapper sysRoleMapper;

    @Resource
    private SysPermissionService sysPermissionService;

    @Resource
    private SysRolePermissionService sysRolePermissionService;

    @Resource
    private SysUserService sysUserService;

    @Resource
    private SysUserRoleService sysUserRoleService;

    @Autowired
    private SysRoleService sysRoleService;

    /**
     * 添加角色
     * @param sysRoleQueryParam
     * @return
     */
    @Override
    public ApiResult addSysRole(SysRoleQueryParam sysRoleQueryParam) {


        List<SysRole> sysRoleManage =sysRoleService.list(new QueryWrapper<SysRole>().orderByDesc("id"));
        //取最大角色id
        SysRole sysRole = new SysRole();
        BeanUtils.copyProperties(sysRoleQueryParam,sysRole);
        if (sysRoleManage.size() > 0) {
            sysRole.setCode("role"+sysRoleManage.get(0).getId());
        }else{
            sysRole.setCode("role1");
        }
        try {
            if (isExistsByCode(sysRole.getCode())) {
                throw new BusinessException("角色编码已存在");
            }
        } catch (Exception e) {
            return ApiResult.error(StatusCode.SYS_USER_ROLE_EXIST);
        }

        sysRole.setState(StateEnum.ENABLE.getKey());
        sysRole.setType(StateEnum.ENABLE.getKey());
        sysRole.setDeleted(StateEnum.DISABLE.getKey());
        this.save(sysRole);
        return ApiResult.ok();
    }

    /**
     * 修改角色
     * @param sysRoleQueryParam
     * @return
     */
    @Override
    public ApiResult updateSysRole(SysRoleQueryParam sysRoleQueryParam) {
        if(null==sysRoleQueryParam.getId()){
            return ApiResult.ok(StatusCode.FORM_SUBMIT_ID_VERIFY);
        }
        SysRole sysRole = this.getById(sysRoleQueryParam.getId());
        if(null==sysRole){
            return ApiResult.error(StatusCode.FORM_SUBMIT_UPDATE_VERIFY);
        }
//        try {
//            if (isExistsByCode(sysRoleQueryParam.getCode())) {
//                return ApiResult.ok(StatusCode.SYS_USER_ROLE_EXIST);
//            }
//        } catch (Exception e) {
//            return ApiResult.ok(StatusCode.SYS_USER_ROLE_EXIST);
//        }
 //       BeanUtils.copyProperties(sysRoleQueryParam,sysRole);
        sysRole.setName(sysRoleQueryParam.getName());
        sysRole.setRemark(sysRoleQueryParam.getRemark());
        sysRole.setUpdateTime(new Date());
        this.saveOrUpdate(sysRole);
        return ApiResult.ok();
    }

    /**
     * 删除角色
     * @param roleId
     * @return
     * @throws Exception
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public ApiResult deleteSysRole(Long roleId) throws Exception {
         if(sysUserRoleService.count(new QueryWrapper<SysUserRole>().eq("role_id",roleId))>0){
             return ApiResult.error(StatusCode.SYS_USER_ROLE_EXIST);
         }
        // 角色权限关系真实删除
        boolean deletePermissionResult = sysRolePermissionService.deleteSysRolePermissionByRoleId(roleId);
        if (!deletePermissionResult) {
            return ApiResult.error(StatusCode.SYS_ROLE_PERMISSION_DELETE);
        }
        // 角色逻辑删除
        boolean deleteRoleResult = removeById(roleId);
        if (!deleteRoleResult) {
            return ApiResult.error(StatusCode.SYS_USER_ROLE_DELETE);
        }
        return ApiResult.ok();
    }

    /**
     * 获取角色列表
     * @return
     */
    @Override
    public ApiResult getRoleList() {
        return ApiResult.ok(sysRoleMapper.getRoleList());
    }









    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean saveSysRole(AddSysRoleParam addSysRoleParam) throws Exception {
        String code = addSysRoleParam.getCode();
        List<String> permissionIds = addSysRoleParam.getPermissionIds();
        // 校验角色标识code唯一性
        if (isExistsByCode(code)) {
            throw new BusinessException("角色编码已存在");
        }
        // 校验权限列表是否存在
        if (!sysPermissionService.isExistsByPermissionIds(permissionIds)) {
            throw new BusinessException("权限id不存在");
        }

        // 保存角色
        SysRole sysRole = SysRoleConvert.INSTANCE.addSysRoleParamToSysRole(addSysRoleParam);
        boolean saveRoleResult = super.save(sysRole);
        if (!saveRoleResult) {
            throw new DaoException("保存角色失败");
        }

        // 保存角色权限
        Long roleId = sysRole.getId();
        boolean saveRolePermissionResult = sysRolePermissionService.saveSysRolePermission(roleId, permissionIds);
        if (!saveRolePermissionResult) {
            throw new DaoException("保存角色权限失败");
        }
        return true;
    }

//    @Transactional(rollbackFor = Exception.class)
//    @Override
//    public boolean updateSysRole(UpdateSysRoleParam updateSysRoleParam) throws Exception {
//        Long roleId = updateSysRoleParam.getId();
//        List<String> permissionIds = updateSysRoleParam.getPermissionIds();
//        // 校验角色是否存在
//        SysRole sysRole = getById(roleId);
//        if (sysRole == null) {
//            throw new BusinessException("该角色不存在");
//        }
//        // 校验权限列表是否存在
//        if (!sysPermissionService.isExistsByPermissionIds(permissionIds)) {
//            throw new BusinessException("权限列表id匹配失败");
//        }
//
//        // 修改角色
//        sysRole.setName(updateSysRoleParam.getName())
//                .setType(updateSysRoleParam.getType())
//                .setRemark(updateSysRoleParam.getRemark())
//                .setState(updateSysRoleParam.getState())
//                .setUpdateTime(new Date());
//        boolean updateResult = updateById(sysRole);
//        if (!updateResult) {
//            throw new DaoException("修改系统角色失败");
//        }
//
//        // 获取之前的权限id集合
//        List<String> beforeList = sysRolePermissionService.getPermissionIdsByRoleId(roleId);
//        // 差集计算
//        // before：1,2,3,4,5,6
//        // after： 1,2,3,4,7,8
//        // 删除5,6 新增7,8
//        // 此处真实删除，去掉deleted字段的@TableLogic注解
//        Set<String> beforeSet = new HashSet<>(beforeList);
//        Set<String> afterSet = new HashSet<>(permissionIds);
//        SetUtils.SetView deleteSet = SetUtils.difference(beforeSet, afterSet);
//        SetUtils.SetView addSet = SetUtils.difference(afterSet, beforeSet);
//        log.debug("deleteSet = " + deleteSet);
//        log.debug("addSet = " + addSet);
//
//        // 删除权限关联
//        UpdateWrapper updateWrapper = new UpdateWrapper();
//        updateWrapper.eq("role_id", roleId);
//        updateWrapper.in("permission_id", deleteSet);
//        boolean deleteResult = sysRolePermissionService.remove(updateWrapper);
//        if (!deleteResult) {
//            throw new DaoException("删除角色权限关系失败");
//        }
//        // 新增权限关联
//        boolean addResult = sysRolePermissionService.saveSysRolePermissionBatch(roleId, addSet);
//        if (!addResult) {
//            throw new DaoException("新增角色权限关系失败");
//        }
//        return true;
//    }



    @Override
    public SysRoleQueryVo getSysRoleById(Serializable id) throws Exception {
        return sysRoleMapper.getSysRoleById(id);
    }

    @Override
    public PageTool<SysRoleQueryVo> getSysRolePageList(SysRoleQueryParam sysRoleQueryParam) throws Exception {
        Page page = setPageParam(sysRoleQueryParam);
        IPage<SysRoleQueryVo> iPage = sysRoleMapper.getSysRolePageList(page, sysRoleQueryParam);
        return new PageTool(iPage);
    }

    @Override
    public boolean isEnableSysRole(Long id) throws Exception {
        SysRole sysRole = new SysRole()
                .setId(id)
                .setState(StateEnum.ENABLE.ordinal());
        int count = sysRoleMapper.selectCount(new QueryWrapper<>(sysRole));
        return count > 0;
    }

    @Override
    public boolean isEnableSysRole(Set<Long> ids) throws Exception {

        for (Long id : ids) {
            SysRole sysRole = new SysRole()
                    .setId(id)
                    .setState(StateEnum.ENABLE.ordinal());
            int count = sysRoleMapper.selectCount(new QueryWrapper<>(sysRole));
            if (count == 0) {
                return false;
            }
        }
        return true;
    }

    @Override
    public boolean isExistsByCode(String code) throws Exception {
        SysRole sysRole = new SysRole().setCode(code);
        return sysRoleMapper.selectCount(new QueryWrapper<>(sysRole)) > 0;
    }

    @Override
    public List<SysRoleQueryVo> getUserRoleByUserId(Long userId) {
        return sysRoleMapper.getUserRoleByUserId(userId);
    }


}
