package com.xdf.creative.shiro.filter;

import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson.JSON;
import com.xdf.creative.base.vo.creative.OauthTokenQueryVo;
import com.xdf.creative.enums.ApiCode;
import com.xdf.creative.module.entity.SysUser;
import com.xdf.creative.module.service.OauthTokenService;
import com.xdf.creative.module.service.SysPermissionService;
import com.xdf.creative.module.service.SysUserService;
import com.xdf.creative.shiro.jwt.JwtUtil;
import com.xdf.creative.util.BeanUtil;
import com.xdf.creative.util.StringUtil;
import com.xdf.creative.util.page.ApiResult;
import org.apache.shiro.web.util.WebUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.PrintWriter;
import java.util.*;

/**
 * @author : lgw
 * @date : 11:00 2020/3/5
 */

@Component
public class ConfigPathInterceptor implements HandlerInterceptor {
    private final static Logger log = LoggerFactory.getLogger(HandlerInterceptor.class);

    @Autowired
    private SysUserService sysUserService;
    @Autowired
    private OauthTokenService oauthTokenService;


    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
       String token = request.getHeader("Authorization");
        if (null == token||StringUtil.isEmpty(token)) {
            ApiResult result = new ApiResult();
            result.setCode(ApiCode.TOKEN_INVALID.getCode());
            result.setMsg(ApiCode.TOKEN_INVALID.getMsg());
            getResponseResult(response, result);
            return false;
        }
        setSysUserServiceBean();
        setOauthTokenServiceBean();
        if(!verifyToken(token,request,response)) return false;
        return true;
    }

    @Override
    public void postHandle(HttpServletRequest request, HttpServletResponse response, Object handler,
                           ModelAndView modelAndView) throws Exception {
    }

    @Override
    public void afterCompletion(HttpServletRequest request, HttpServletResponse response, Object handler, Exception ex)
            throws Exception {
    }



/**
     * 验证token
     *
     * @param token
     * @return
     */

    public boolean verifyToken(String token,HttpServletRequest request, HttpServletResponse response) {
        String requestUrl = request.getRequestURI();
        ApiResult result = new ApiResult();
        //token失效
        if (JwtUtil.isExpired(token)) {
            log.error("令牌过期");
            result.setCode(ApiCode.TOKEN_INVALID.getCode());
            result.setMsg(ApiCode.TOKEN_INVALID.getMsg());
            getResponseResult(response, result);
            return  false;
        }
        String userName = JwtUtil.getUsername(token);
        Long userId = JwtUtil.getUserId(token);
        if (StringUtil.isEmpty(userName) || null==userId) {
            log.error(ApiCode.TOKEN_INVALID.getMsg());
            result.setCode(ApiCode.TOKEN_INVALID.getCode());
            result.setMsg(ApiCode.TOKEN_INVALID.getMsg());
            getResponseResult(response, result);
            return  false;
        }
        //先查token是否存在
        OauthTokenQueryVo oauthTokenQueryVo =oauthTokenService.getOauthTokenByToken(token);
        if(null==oauthTokenQueryVo){
            log.error("该token未登录，非法访问!");
            result.setCode(ApiCode.TOKEN_INVALID.getCode());
            result.setMsg(ApiCode.TOKEN_INVALID.getMsg());
            getResponseResult(response, result);
            return false;
        }
        //根据用户名去数据库查询用户信息
        SysUser user = null;
        //获取用户信息
        try {
            user = sysUserService.getSysUserByUserName(userName);
        } catch (Exception e) {
            log.error("用户 { " + userName + " } 不存在 ");
            result.setCode(ApiCode.USER_EXIST.getCode());
            result.setMsg(ApiCode.USER_EXIST.getMsg());
            getResponseResult(response, result);
            return false;
        }
        if (user == null) {
            //这里返回后会报出对应异常
            result.setCode(ApiCode.USER_EXIST.getCode());
            result.setMsg(ApiCode.USER_EXIST.getMsg());
            getResponseResult(response, result);
            return false;
        }
       /* List<String> permissionList = Arrays.asList(oauthTokenQueryVo.getPermissions().trim().split(","));
        boolean permissionflag = false;
        for (String permissionUrl:permissionList ) {
            if(permissionUrl.contains(requestUrl)){
               return true;
            }
        }
        if(!permissionflag){
            log.error("该用户没有权限访问"+requestUrl);
            result.setCode(ApiCode.NOT_PERMISSION.getCode());
            result.setMsg(ApiCode.NOT_PERMISSION.getMsg());
            getResponseResult(response, result);
            return false;
        }*/

//        if (!permissionList.contains(requestUrl)) {
//            log.error("该用户没有权限访问"+requestUrl);
//            result.setCode(ApiCode.NOT_PERMISSION.getCode());
//            result.setMsg(ApiCode.NOT_PERMISSION.getMsg());
//            getResponseResult(response, result);
//            return false;
//        }
        return true;
    }
/*
*
     * 返回结果
     *
     * @param response
     */

    public void getResponseResult(ServletResponse response, ApiResult result) {
        response.setCharacterEncoding("UTF-8");
        response.setContentType("application/json;charset=utf-8");
        PrintWriter printWriter = null;
        try {
            printWriter = WebUtils.toHttp(response).getWriter();
            printWriter.write(JSON.toJSONString(result));
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        } finally {
            if (null != printWriter) {
                printWriter.close();
            }
        }
    }

    public void setSysUserServiceBean() {
        if (null == sysUserService) {
            sysUserService = BeanUtil.getBean(SysUserService.class);
        }
    }

    public void setOauthTokenServiceBean() {
        if (null == oauthTokenService) {
            oauthTokenService = BeanUtil.getBean(OauthTokenService.class);
        }
    }
}


